@@ -1,13 +1,17 @@
 /*
  * The SIP module interface.
  *
- * Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
- * 
+ * Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+ *
  * This file is part of SIP.
- * 
+ *
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- * 
+ *
+ * This copy of SIP may also used under the terms of the GNU General Public
+ * License v2 or v3 as published by the Free Software Foundation which can be
+ * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+ *
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
@@ -50,8 +54,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040802
-#define SIP_VERSION_STR     "4.8.2"
+#define SIP_VERSION         0x040a02
+#define SIP_VERSION_STR     "4.10.2"
 
 
 /*
@@ -64,6 +68,28 @@ extern "C" {
  *
  * History:
  *
+ * 7.1  Added the 'H' format character to sip_api_parse_result().
+ *      Deprecated the 'D' format character of sip_api_parse_result().
+ *
+ * 7.0  Added sip_api_parse_kwd_args().
+ *      Added sipErrorState, sip_api_add_exception().
+ *      The type initialisation function is now passed a dictionary of keyword
+ *      arguments.
+ *      All argument parsers now update a set of error messages rather than an
+ *      argument count.
+ *      The signatures of sip_api_no_function() and sip_api_no_method() have
+ *      changed.
+ *      Added ctd_docstring to sipClassTypeDef.
+ *      Added vf_docstring to sipVersionedFunctionDef.
+ *
+ * 6.0  Added the sipContainerDef structure to define the contents of a class
+ *      or mapped type.  Restructured sipClassDef and sipMappedTypeDef
+ *      accordingly.
+ *      Added the 'r' format character to sip_api_parse_args().
+ *      Added the 'r' format character to sip_api_call_method() and
+ *      sip_api_build_result().
+ *      Added the assignment, array and copy allocation helpers.
+ *
  * 5.0  Added sip_api_is_api_enabled().
  *      Renamed the td_version_nr member of sipTypeDef to be int and where -1
  *      indicates it is not versioned.
@@ -142,8 +168,8 @@ extern "C" {
  *
  * 0.0  Original version.
  */
-#define SIP_API_MAJOR_NR    5
-#define SIP_API_MINOR_NR    0
+#define SIP_API_MAJOR_NR    7
+#define SIP_API_MINOR_NR    1
 
 
 /* Some compatibility stuff to help with handwritten code for SIP v3. */
@@ -327,8 +353,8 @@ typedef struct _sipEnumTypeObject {
 /*
  * Some convenient function pointers.
  */
-typedef void *(*sipInitFunc)(sipSimpleWrapper *, PyObject *, PyObject **,
-        int *);
+typedef void *(*sipInitFunc)(sipSimpleWrapper *, PyObject *, PyObject *,
+        PyObject **, PyObject **, PyObject **);
 typedef int (*sipTraverseFunc)(void *, visitproc, void *);
 typedef int (*sipClearFunc)(void *);
 #if PY_MAJOR_VERSION >= 3
@@ -344,6 +370,9 @@ typedef const struct _sipTypeDef *(*sipSubClassConvertFunc)(void **);
 typedef int (*sipConvertToFunc)(PyObject *, void **, int *, PyObject *);
 typedef PyObject *(*sipConvertFromFunc)(void *, PyObject *);
 typedef int (*sipVirtHandlerFunc)(void *, PyObject *, ...);
+typedef void (*sipAssignFunc)(void *, SIP_SSIZE_T, const void *);
+typedef void *(*sipArrayFunc)(SIP_SSIZE_T);
+typedef void *(*sipCopyFunc)(const void *, SIP_SSIZE_T);
 typedef void (*sipReleaseFunc)(void *, int);
 typedef PyObject *(*sipPickleFunc)(void *);
 typedef int (*sipAttrGetterFunc)(const struct _sipTypeDef *, PyObject *);
@@ -352,18 +381,18 @@ typedef int (*sipVariableSetterFunc)(void *, PyObject *, PyObject *);
 
 
 /*
- * The information describing an encoded class ID.
+ * The information describing an encoded type ID.
  */
-typedef struct _sipEncodedClassDef {
-    /* The class number. */
-    unsigned sc_class:16;
+typedef struct _sipEncodedTypeDef {
+    /* The type number. */
+    unsigned sc_type:16;
 
     /* The module number (255 for this one). */
     unsigned sc_module:8;
 
     /* A context specific flag. */
     unsigned sc_flag:1;
-} sipEncodedClassDef;
+} sipEncodedTypeDef;
 
 
 /*
@@ -421,11 +450,14 @@ typedef struct _sipInstancesDef {
  * The information describing a type initialiser extender.
  */
 typedef struct _sipInitExtenderDef {
+    /* The API version range index. */
+    int ie_api_range;
+
     /* The extender function. */
     sipInitFunc ie_extender;
 
     /* The class being extended. */
-    sipEncodedClassDef ie_class;
+    sipEncodedTypeDef ie_class;
 
     /* The next extender for this class. */
     struct _sipInitExtenderDef *ie_next;
@@ -440,13 +472,23 @@ typedef struct _sipSubClassConvertorDef {
     sipSubClassConvertFunc scc_convertor;
 
     /* The encoded base type. */
-    sipEncodedClassDef scc_base;
+    sipEncodedTypeDef scc_base;
 
     /* The base type. */
     struct _sipTypeDef *scc_basetype;
 } sipSubClassConvertorDef;
 
 
+/*
+ * The different error states of handwritten code.
+ */
+typedef enum {
+    sipErrorNone,       /* There is no error. */
+    sipErrorFail,       /* The error is a failure. */
+    sipErrorContinue    /* It may not apply if a later operation succeeds. */
+} sipErrorState;
+
+
 /*
  * The different Python slot types.
  */
@@ -510,6 +552,8 @@ typedef enum {
 #if PY_VERSION_HEX >= 0x02050000
     index_slot,         /* __index__ */
 #endif
+    iter_slot,          /* __iter__ */
+    next_slot,          /* __next__ */
 } sipPySlotType;
 
 
@@ -536,7 +580,7 @@ typedef struct _sipPySlotExtenderDef {
     sipPySlotType pse_type;
 
     /* The encoded class. */
-    sipEncodedClassDef pse_class;
+    sipEncodedTypeDef pse_class;
 } sipPySlotExtenderDef;
 
 
@@ -575,7 +619,7 @@ typedef struct _sipVariableDef {
  * namespace, a mapped type or a named enum.
  */
 typedef struct _sipTypeDef {
-    /* The version number, -1 if the type isn't versioned. */
+    /* The version range index, -1 if the type isn't versioned. */
     int td_version;
 
     /* The next version of this type. */
@@ -601,6 +645,46 @@ typedef struct _sipTypeDef {
 } sipTypeDef;
 
 
+/*
+ * The information describing a container (ie. a class, namespace or a mapped
+ * type).
+ */
+typedef struct _sipContainerDef {
+    /*
+     * The Python name of the type, -1 if this is a namespace extender (in the
+     * context of a class) or doesn't require a namespace (in the context of a
+     * mapped type). */
+    int cod_name;
+
+    /*
+     * The scoping type or the namespace this is extending if it is a namespace
+     * extender.
+     */
+    sipEncodedTypeDef cod_scope;
+
+    /* The number of lazy methods. */
+    int cod_nrmethods;
+
+    /* The table of lazy methods. */
+    PyMethodDef *cod_methods;
+
+    /* The number of lazy enum members. */
+    int cod_nrenummembers;
+
+    /* The table of lazy enum members. */
+    sipEnumMemberDef *cod_enummembers;
+
+    /* The number of variables. */
+    int cod_nrvariables;
+
+    /* The table of variables. */
+    sipVariableDef *cod_variables;
+
+    /* The static instances. */
+    sipInstancesDef cod_instances;
+} sipContainerDef;
+
+
 /*
  * The information describing a C++ class (or C struct) or a C++ namespace.
  */
@@ -608,8 +692,11 @@ typedef struct _sipClassTypeDef {
     /* The base type information. */
     sipTypeDef ctd_base;
 
-    /* The Python name of the type, -1 if this is a namespace extender. */
-    int ctd_name;
+    /* The container information. */
+    sipContainerDef ctd_container;
+
+    /* The docstring. */
+    const char *ctd_docstring;
 
     /*
      * The meta-type name, -1 to use the meta-type of the first super-type
@@ -620,36 +707,12 @@ typedef struct _sipClassTypeDef {
     /* The super-type name, -1 to use sipWrapper. */
     int ctd_supertype;
 
-    /*
-     * The scoping type or the namespace this is extending if it is a namespace
-     * extender.
-     */
-    sipEncodedClassDef ctd_scope;
-
     /* The super-types. */
-    sipEncodedClassDef *ctd_supers;
+    sipEncodedTypeDef *ctd_supers;
 
     /* The table of Python slots. */
     sipPySlotDef *ctd_pyslots;
 
-    /* The number of lazy methods. */
-    int ctd_nrmethods;
-
-    /* The table of lazy methods. */
-    PyMethodDef *ctd_methods;
-
-    /* The number of lazy enum members. */
-    int ctd_nrenummembers;
-
-    /* The table of lazy enum members. */
-    sipEnumMemberDef *ctd_enummembers;
-
-    /* The number of variables. */
-    int ctd_nrvariables;
-
-    /* The table of variables. */
-    sipVariableDef *ctd_variables;
-
     /* The initialisation function. */
     sipInitFunc ctd_init;
 
@@ -682,6 +745,15 @@ typedef struct _sipClassTypeDef {
     /* The deallocation function. */
     sipDeallocFunc ctd_dealloc;
 
+    /* The optional assignment function. */
+    sipAssignFunc ctd_assign;
+
+    /* The optional array allocation function. */
+    sipArrayFunc ctd_array;
+
+    /* The optional copy function. */
+    sipCopyFunc ctd_copy;
+
     /* The release function, 0 if a C strict. */
     sipReleaseFunc ctd_release;
 
@@ -691,9 +763,6 @@ typedef struct _sipClassTypeDef {
     /* The optional convert to function. */
     sipConvertToFunc ctd_cto;
 
-    /* The static instances. */
-    sipInstancesDef ctd_instances;
-
     /* The next namespace extender. */
     struct _sipClassTypeDef *ctd_nsextender;
 
@@ -709,6 +778,18 @@ typedef struct _sipMappedTypeDef {
     /* The base type information. */
     sipTypeDef mtd_base;
 
+    /* The container information. */
+    sipContainerDef mtd_container;
+
+    /* The optional assignment function. */
+    sipAssignFunc mtd_assign;
+
+    /* The optional array allocation function. */
+    sipArrayFunc mtd_array;
+
+    /* The optional copy function. */
+    sipCopyFunc mtd_copy;
+
     /* The optional release function. */
     sipReleaseFunc mtd_release;
 
@@ -775,6 +856,29 @@ typedef struct _sipDelayedDtor {
 } sipDelayedDtor;
 
 
+/*
+ * Defines an entry in the table of global functions all of whose overloads
+ * are versioned (so their names can't be automatically added to the module
+ * dictionary).
+ */
+typedef struct _sipVersionedFunctionDef {
+    /* The name, -1 marks the end of the table. */
+    int vf_name;
+
+    /* The function itself. */
+    PyCFunction vf_function;
+
+    /* The METH_* flags. */
+    int vf_flags;
+
+    /* The docstring. */
+    const char *vf_docstring;
+
+    /* The API version range index. */
+    int vf_api_range;
+} sipVersionedFunctionDef;
+
+
 /*
  * The information describing an imported module.
  */
@@ -868,12 +972,15 @@ typedef struct _sipExportedModuleDef {
 
     /*
      * The array of API version definitions.  Each definition takes up 3
-     * elements.
+     * elements.  If the third element of a 3-tuple is negative then the first
+     * two elements define an API and its default version.  All such
+     * definitions will appear at the end of the array.  If the first element
+     * of a 3-tuple is negative then that is the last element of the array.
      */
     int *em_versions;
 
     /* The optional table of versioned functions. */
-    void *em_versioned_functions;
+    sipVersionedFunctionDef *em_versioned_functions;
 } sipExportedModuleDef;
 
 
@@ -1173,6 +1280,7 @@ typedef struct _sipAPIDef {
     int (*api_register_attribute_getter)(const sipTypeDef *td,
             sipAttrGetterFunc getter);
     int (*api_is_api_enabled)(const char *name, int from, int to);
+    sipErrorState (*api_bad_callable_arg)(int arg_nr, PyObject *arg);
 
     /*
      * The following are deprecated parts of the public API.
@@ -1203,14 +1311,15 @@ typedef struct _sipAPIDef {
      * The following are not part of the public API.
      */
     int (*api_init_module)(sipExportedModuleDef *client, PyObject *mod_dict);
-    int (*api_parse_args)(int *argsParsedp, PyObject *sipArgs,
+    int (*api_parse_args)(PyObject **parseErrp, PyObject *sipArgs,
             const char *fmt, ...);
-    int (*api_parse_pair)(int *argsParsedp, PyObject *arg0, PyObject *arg1,
+    int (*api_parse_pair)(PyObject **parseErrp, PyObject *arg0, PyObject *arg1,
             const char *fmt, ...);
     void (*api_common_dtor)(sipSimpleWrapper *sipSelf);
-    void (*api_no_function)(int argsParsed, const char *func);
-    void (*api_no_method)(int argsParsed, const char *classname,
-            const char *method);
+    void (*api_no_function)(PyObject *parseErr, const char *func,
+            const char *doc);
+    void (*api_no_method)(PyObject *parseErr, const char *scope,
+            const char *method, const char *doc);
     void (*api_abstract_method)(const char *classname, const char *method);
     void (*api_bad_class)(const char *classname);
     void *(*api_get_cpp_ptr)(sipSimpleWrapper *w, const sipTypeDef *td);
@@ -1246,6 +1355,10 @@ typedef struct _sipAPIDef {
 #endif
     int (*api_deprecated)(const char *classname, const char *method);
     void (*api_keep_reference)(PyObject *self, int key, PyObject *obj);
+    int (*api_parse_kwd_args)(PyObject **parseErrp, PyObject *sipArgs,
+            PyObject *sipKwdArgs, const char **kwdlist, PyObject **unused,
+            const char *fmt, ...);
+    void (*api_add_exception)(sipErrorState es, PyObject **parseErrp);
 } sipAPIDef;
 
 
@@ -1329,6 +1442,7 @@ typedef struct _sipQtAPI {
 #define SIP_TYPE_ABSTRACT   0x0008  /* If the type is abstract. */
 #define SIP_TYPE_SCC        0x0010  /* If the type is subject to sub-class convertors. */
 #define SIP_TYPE_ALLOW_NONE 0x0020  /* If the type can handle None. */
+#define SIP_TYPE_STUB       0x0040  /* If the type is a stub. */
 
 
 /*
@@ -1360,13 +1474,15 @@ typedef struct _sipQtAPI {
 #define sipTypeIsAbstract(td)   ((td)->td_flags & SIP_TYPE_ABSTRACT)
 #define sipTypeHasSCC(td)   ((td)->td_flags & SIP_TYPE_SCC)
 #define sipTypeAllowNone(td)    ((td)->td_flags & SIP_TYPE_ALLOW_NONE)
+#define sipTypeIsStub(td)   ((td)->td_flags & SIP_TYPE_STUB)
+#define sipTypeSetStub(td)  ((td)->td_flags |= SIP_TYPE_STUB)
 
 /*
  * Get various names from the string pool for various data types.
  */
 #define sipNameFromPool(em, mr) (&((em)->em_strings)[(mr)])
 #define sipNameOfModule(em)     sipNameFromPool((em), (em)->em_name)
-#define sipPyNameOfClass(ctd)   sipNameFromPool((ctd)->ctd_base.td_module, (ctd)->ctd_name)
+#define sipPyNameOfContainer(cod, td)   sipNameFromPool((td)->td_module, (cod)->cod_name)
 #define sipPyNameOfEnum(etd)    sipNameFromPool((etd)->etd_base.td_module, (etd)->etd_name)
 
 
@@ -1417,6 +1533,9 @@ typedef struct _pyqt4QtSignal {
     /* The C++ name and signature of the signal. */
     const char *signature;
 
+    /* The optional docstring. */
+    const char *docstring;
+
     /*
      * If the signal is an overload of regular methods then this points to the
      * code that implements those methods.
@@ -1435,9 +1554,6 @@ typedef struct _pyqt4ClassTypeDef {
      */
     sipClassTypeDef super;
 
-    /* A pointer to the optional assignment helper function. */
-    void (*qt4_assign_func)(void *, const void *);
-
     /* A pointer to the QObject sub-class's staticMetaObject class variable. */
     const void *qt4_static_metaobject;
 
@@ -1455,22 +1571,6 @@ typedef struct _pyqt4ClassTypeDef {
 } pyqt4ClassTypeDef;
 
 
-/*
- * This is the PyQt4-specific extension to the generated mapped type structure.
- * In SIP v5 this will be pushed out to a plugin supplied by PyQt4.
- */
-typedef struct _pyqt4MappedTypeDef {
-    /*
-     * The super-type structure.  This must be first in the structure so that
-     * it can be cast to sipMappedTypeDef *.
-     */
-    sipMappedTypeDef super;
-
-    /* A pointer to the optional assignment helper function. */
-    void (*qt4_assign_func)(void *, const void *);
-} pyqt4MappedTypeDef;
-
-
 #ifdef __cplusplus
 }
 #endif
